/*
Tremolo Techno to forget the pain of life  
July 2020 
*/

#include <Bela.h>
#include <cmath>
#include <algorithm>
#include <libraries/Scope/Scope.h>

int gAudioChannelNum; // number of audio channels to iterate over
float gFrequency = 4.0;
float gPhase;
float gInverseSampleRate;
int gAudioFramesPerAnalogFrame = 0;


// Browser-based oscilloscope
Scope gScope;

// Rebus specific Gain and Phase 
float gMinPhase = 0.01;
float gMaxPhase = 0.44;
float gMidPhase = 0.2;
float gMinGain = 0.150467;
float gMaxGain =  0.3;


bool setup(BelaContext *context, void *userData)
{
	// If the amout of audio input and output channels is not the same
	// we will use the minimum between input and output
	gAudioChannelNum = std::min(context->audioInChannels, context->audioOutChannels);
	
	// Check that we have the same number of inputs and outputs.
	if(context->audioInChannels != context->audioOutChannels){
		printf("Different number of audio outputs and inputs available. Using %d channels.\n", gAudioChannelNum);
	}

	if(context->analogSampleRate > context->audioSampleRate)
	{
		fprintf(stderr, "Error: for this project the sampling rate of the analog inputs has to be <= the audio sample rate\n");
		return false;
	}

	if(context->audioFrames)
		gAudioFramesPerAnalogFrame = context->audioFrames / context->analogFrames;
	gInverseSampleRate = 1.0 / context->audioSampleRate;
	gPhase = 0.0;
	
	// Set up the oscilloscope
	gScope.setup(5, context->audioSampleRate);
	
	return true;
}

void render(BelaContext *context, void *userData)
{
	// Nested for loops for audio channels
	for(unsigned int n = 0; n < context->audioFrames; n++) {


    	// read Gain and Phase
    	float phaseReading = analogRead(context, n/2, 0);
    	float gainReading = analogRead(context, n/2, 4);
    	
    	float amplitude = map(gainReading, gMinGain, gMaxGain, 0, 1);
		amplitude = constrain(amplitude, 0, 1);
		
		 float frequency = map(phaseReading, gMinPhase, gMaxPhase, 100, 1000);
		frequency = constrain(frequency, 50.0, 1000.0);

		// Generate a sinewave with frequency set by gFrequency
		// and amplitude from -0.5 to 0.5
		float lfo = sinf(gPhase) * 0.5f;
		// Keep track and wrap the phase of the sinewave
		gPhase += 2.0f * (float)M_PI * frequency * gInverseSampleRate;
		if(gPhase > M_PI)
			gPhase -= 2.0f * (float)M_PI;

		for(unsigned int channel = 0; channel < gAudioChannelNum; channel++) {
			// Read the audio input and half the amplitude
			float input = audioRead(context, n, channel) * 0.5f;
			
			float out = (input*lfo) * amplitude;
			
			// Write to audio output the audio input multiplied by the sinewave
			audioWrite(context, n, channel, out);
			
		// Write the output to the oscilloscope
    	gScope.log(lfo, input, out,  gainReading, phaseReading); 
    	
		}
	}
}

void cleanup(BelaContext *context, void *userData)
{

}


/**
\example tremolo/render.cpp

A simple tremolo effect
-----------------------

This sketch demonstrates how to make a simple tremolo effect with one potiometer to
control the rate of the effect. A tremolo effect is a simple type of amplitude modulation
where the amplitude of one signal is continuous modulated by the amplitude of another.
This is achieved by multiplying two signals together.

In this example we want to create a tremolo effect like that you would find in a guitar
effects box so our first signal will be our audio input into which we could plug a guitar
or external sound source. This will be our 'carrier' signal.

The second signal that we will use, the 'modulator', will be a low freqeuncy oscillator (LFO),
in this case a sinetone which we will generate in the same way as the 01-Basic/sinetone example. 
The frequency of this sinetone is determined by a global variable, `gFrequency`. Again, the 
sinetone is produced by incrementing the phase of a sine function on every audio frame.

In `render()` you'll see two nested for loop structures, one for audio and the other for the 
analogs. You should be pretty familiar with this structure by now. In the first of these loops
we deal with all the audio -- in the second with reading the analog input channels. We read the 
value of analog input 0 and map it to an appropriate range for controlling the frequency
of the sine tone.

The lfo is then mulitplied together with the audio input and sent to the audio output.

Hardware
-----------------------
- connect a 10K pot to 3.3V and GND on its 1st and 3rd pins.
- connect the 2nd middle pin of the pot to analogIn 0.

*/
